"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.parallel = exports.isInteresting = exports.StopWatch = exports.containsLocation = exports.containsRange = exports.containsPositionStrict = exports.containsPosition = exports.compareRangeByStart = exports.isBefore = exports.isBeforeOrEqual = exports.nodeAtPosition = exports.identifierAtPosition = exports.asLspRange = exports.LanguageInfo = exports.symbolMapping = void 0;
const lsp = __importStar(require("vscode-languageserver"));
exports.symbolMapping = new class {
    constructor() {
        this._symbolKindMapping = new Map([
            ['file', lsp.SymbolKind.File],
            ['module', lsp.SymbolKind.Module],
            ['namespace', lsp.SymbolKind.Namespace],
            ['package', lsp.SymbolKind.Package],
            ['class', lsp.SymbolKind.Class],
            ['method', lsp.SymbolKind.Method],
            ['property', lsp.SymbolKind.Property],
            ['field', lsp.SymbolKind.Field],
            ['constructor', lsp.SymbolKind.Constructor],
            ['enum', lsp.SymbolKind.Enum],
            ['interface', lsp.SymbolKind.Interface],
            ['function', lsp.SymbolKind.Function],
            ['variable', lsp.SymbolKind.Variable],
            ['constant', lsp.SymbolKind.Constant],
            ['string', lsp.SymbolKind.String],
            ['number', lsp.SymbolKind.Number],
            ['boolean', lsp.SymbolKind.Boolean],
            ['array', lsp.SymbolKind.Array],
            ['object', lsp.SymbolKind.Object],
            ['key', lsp.SymbolKind.Key],
            ['null', lsp.SymbolKind.Null],
            ['enumMember', lsp.SymbolKind.EnumMember],
            ['struct', lsp.SymbolKind.Struct],
            ['event', lsp.SymbolKind.Event],
            ['operator', lsp.SymbolKind.Operator],
            ['typeParameter', lsp.SymbolKind.TypeParameter],
        ]);
    }
    getSymbolKind(symbolKind, strict) {
        const res = this._symbolKindMapping.get(symbolKind);
        if (!res && strict) {
            return undefined;
        }
        return res ?? lsp.SymbolKind.Variable;
    }
};
class LanguageInfo {
    constructor(languageId, wasmUri, suffixes) {
        this.languageId = languageId;
        this.wasmUri = wasmUri;
        this.suffixes = suffixes;
    }
}
exports.LanguageInfo = LanguageInfo;
;
// --- geometry
function asLspRange(node) {
    return lsp.Range.create(node.startPosition.row, node.startPosition.column, node.endPosition.row, node.endPosition.column);
}
exports.asLspRange = asLspRange;
function identifierAtPosition(identQuery, node, position) {
    // `foo|::bar` -> finds `::`
    let candidate = nodeAtPosition(node, position, false);
    let capture = identQuery.captures(candidate);
    if (capture.length === 1) {
        return candidate;
    }
    // `foo|::bar` -> finds `foo`
    candidate = nodeAtPosition(node, position, true);
    capture = identQuery.captures(candidate);
    if (capture.length === 1) {
        return candidate;
    }
    return undefined;
}
exports.identifierAtPosition = identifierAtPosition;
function nodeAtPosition(node, position, leftBias = false) {
    for (const child of node.children) {
        const range = asLspRange(child);
        if (isBeforeOrEqual(range.start, position)) {
            if (isBefore(position, range.end)) {
                return nodeAtPosition(child, position, leftBias);
            }
            if (leftBias && isBeforeOrEqual(position, range.end)) {
                return nodeAtPosition(child, position, leftBias);
            }
        }
    }
    return node;
}
exports.nodeAtPosition = nodeAtPosition;
function isBeforeOrEqual(a, b) {
    if (a.line < b.line) {
        return true;
    }
    if (b.line < a.line) {
        return false;
    }
    return a.character <= b.character;
}
exports.isBeforeOrEqual = isBeforeOrEqual;
function isBefore(a, b) {
    if (a.line < b.line) {
        return true;
    }
    if (b.line < a.line) {
        return false;
    }
    return a.character < b.character;
}
exports.isBefore = isBefore;
function compareRangeByStart(a, b) {
    if (isBefore(a.start, b.start)) {
        return -1;
    }
    else if (isBefore(b.start, a.start)) {
        return 1;
    }
    // same start...
    if (isBefore(a.end, b.end)) {
        return -1;
    }
    else if (isBefore(b.end, a.end)) {
        return 1;
    }
    return 0;
}
exports.compareRangeByStart = compareRangeByStart;
function containsPosition(range, position) {
    return isBeforeOrEqual(range.start, position) && isBeforeOrEqual(position, range.end);
}
exports.containsPosition = containsPosition;
function containsPositionStrict(range, position) {
    return isBefore(range.start, position) && isBefore(position, range.end);
}
exports.containsPositionStrict = containsPositionStrict;
function containsRange(range, other) {
    return containsPosition(range, other.start) && containsPosition(range, other.end);
}
exports.containsRange = containsRange;
function containsLocation(loc, uri, position) {
    return loc.uri === uri && containsPosition(loc.range, position);
}
exports.containsLocation = containsLocation;
class StopWatch {
    constructor() {
        this.t1 = performance.now();
    }
    reset() {
        this.t1 = performance.now();
    }
    elapsed() {
        return (performance.now() - this.t1).toFixed(2);
    }
}
exports.StopWatch = StopWatch;
function isInteresting(uri) {
    return !/^(git|github|vsls):/i.test(uri);
}
exports.isInteresting = isInteresting;
async function parallel(tasks, degree, token) {
    let result = [];
    let pos = 0;
    while (true) {
        if (token.isCancellationRequested) {
            throw new Error('cancelled');
        }
        const partTasks = tasks.slice(pos, pos + degree);
        if (partTasks.length === 0) {
            break;
        }
        const partResult = await Promise.all(partTasks.map(task => task(token)));
        pos += degree;
        result.push(...partResult);
    }
    return result;
}
exports.parallel = parallel;
