"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DocumentStore = void 0;
const lsp = __importStar(require("vscode-languageserver"));
const vscode_languageserver_1 = require("vscode-languageserver");
const vscode_languageserver_textdocument_1 = require("vscode-languageserver-textdocument");
const languages_1 = __importDefault(require("./languages"));
const lruMap_1 = require("./util/lruMap");
class DocumentStore extends vscode_languageserver_1.TextDocuments {
    constructor(_connection) {
        super({
            create: vscode_languageserver_textdocument_1.TextDocument.create,
            update: (doc, changes, version) => {
                let result;
                let incremental = true;
                let event = { document: doc, changes: [] };
                for (const change of changes) {
                    if (!lsp.TextDocumentContentChangeEvent.isIncremental(change)) {
                        incremental = false;
                        break;
                    }
                    const rangeOffset = doc.offsetAt(change.range.start);
                    event.changes.push({
                        text: change.text,
                        range: change.range,
                        rangeOffset,
                        rangeLength: change.rangeLength ?? doc.offsetAt(change.range.end) - rangeOffset,
                    });
                }
                result = vscode_languageserver_textdocument_1.TextDocument.update(doc, changes, version);
                if (incremental) {
                    this._onDidChangeContent2.fire(event);
                }
                return result;
            }
        });
        this._connection = _connection;
        this._onDidChangeContent2 = new lsp.Emitter();
        this.onDidChangeContent2 = this._onDidChangeContent2.event;
        this._decoder = new TextDecoder();
        this._fileDocuments = new lruMap_1.LRUMap({
            size: 200,
            dispose: _entries => { }
        });
        super.listen(_connection);
        _connection.onNotification('file-cache/remove', uri => this._fileDocuments.delete(uri));
    }
    async retrieve(uri) {
        let result = this.get(uri);
        if (result) {
            return result;
        }
        let promise = this._fileDocuments.get(uri);
        if (!promise) {
            promise = this._requestDocument(uri);
            this._fileDocuments.set(uri, promise);
        }
        return promise;
    }
    async _requestDocument(uri) {
        const reply = await this._connection.sendRequest('file/read', uri);
        return vscode_languageserver_textdocument_1.TextDocument.create(uri, languages_1.default.getLanguageIdByUri(uri), 1, this._decoder.decode(reply));
    }
}
exports.DocumentStore = DocumentStore;
