"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CompletionItemProvider = void 0;
const lsp = __importStar(require("vscode-languageserver"));
const languages_1 = __importDefault(require("../languages"));
class CompletionItemProvider {
    constructor(_documents, _trees, _symbols) {
        this._documents = _documents;
        this._trees = _trees;
        this._symbols = _symbols;
    }
    register(connection) {
        connection.client.register(lsp.CompletionRequest.type, { documentSelector: languages_1.default.getSupportedLanguages('completions', ['identifiers', 'outline']) });
        connection.onRequest(lsp.CompletionRequest.type, this.provideCompletionItems.bind(this));
    }
    async provideCompletionItems(params) {
        const document = await this._documents.retrieve(params.textDocument.uri);
        const tree = this._trees.getParseTree(document);
        if (!tree) {
            return [];
        }
        const result = new Map();
        // (1) all identifiers that are used in this file
        const query = languages_1.default.getQuery(document.languageId, 'identifiers');
        const captures = query.captures(tree.rootNode);
        for (let capture of captures) {
            const text = capture.node.text;
            result.set(text, { label: text });
        }
        // (2) all definitions that are known in this project (override less specific local identifiers)
        // don't wait for the whole index to be updated but use what we have right now,
        // it is very likely that the current has changed and that we already processed
        // await this._symbols.update();
        for (let [key, symbols] of this._symbols.definitions) {
            const [first] = symbols;
            result.set(key, {
                label: key,
                kind: CompletionItemProvider._kindMapping.get(first.kind)
            });
        }
        return Array.from(result.values());
    }
}
exports.CompletionItemProvider = CompletionItemProvider;
CompletionItemProvider._kindMapping = new Map([
    [lsp.SymbolKind.Class, lsp.CompletionItemKind.Class],
    [lsp.SymbolKind.Interface, lsp.CompletionItemKind.Interface],
    [lsp.SymbolKind.Field, lsp.CompletionItemKind.Field],
    [lsp.SymbolKind.Property, lsp.CompletionItemKind.Property],
    [lsp.SymbolKind.Event, lsp.CompletionItemKind.Event],
    [lsp.SymbolKind.Constructor, lsp.CompletionItemKind.Constructor],
    [lsp.SymbolKind.Method, lsp.CompletionItemKind.Method],
    [lsp.SymbolKind.Enum, lsp.CompletionItemKind.Enum],
    [lsp.SymbolKind.EnumMember, lsp.CompletionItemKind.EnumMember],
    [lsp.SymbolKind.Function, lsp.CompletionItemKind.Function],
    [lsp.SymbolKind.Variable, lsp.CompletionItemKind.Variable],
]);
