"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DefinitionProvider = void 0;
const lsp = __importStar(require("vscode-languageserver"));
const locals_1 = require("./locals");
const common_1 = require("../common");
const languages_1 = __importDefault(require("../languages"));
class DefinitionProvider {
    constructor(_documents, _trees, _symbols) {
        this._documents = _documents;
        this._trees = _trees;
        this._symbols = _symbols;
    }
    register(connection) {
        connection.client.register(lsp.DefinitionRequest.type, { documentSelector: languages_1.default.getSupportedLanguages('definitions', ['locals', 'outline']) });
        connection.onRequest(lsp.DefinitionRequest.type, this.provideDefinitions.bind(this));
    }
    async provideDefinitions(params) {
        const document = await this._documents.retrieve(params.textDocument.uri);
        // find definition in file
        const info = locals_1.Locals.create(document, this._trees);
        const scope = info.root.findScope(params.position);
        const anchor = scope.findDefinitionOrUsage(params.position);
        if (anchor) {
            // find definition inside this file
            const definitions = scope.findDefinitions(anchor.name);
            if (definitions.length > 0) {
                return definitions.map(def => lsp.Location.create(document.uri, def.range));
            }
        }
        // find definition globally
        const tree = this._trees.getParseTree(document);
        if (!tree) {
            return [];
        }
        const query = languages_1.default.getQuery(document.languageId, 'identifiers');
        const candidate = common_1.identifierAtPosition(query, tree.rootNode, params.position);
        if (!candidate) {
            // not an identifier
            return [];
        }
        await this._symbols.update();
        let sameLanguageOffset = 0;
        const result = [];
        const all = this._symbols.definitions.get(candidate.text) ?? [];
        for (const symbol of all) {
            const isSameLanguage = languages_1.default.getLanguageIdByUri(symbol.location.uri) === document.languageId;
            if (isSameLanguage) {
                result.unshift(symbol.location);
                sameLanguageOffset++;
            }
            else {
                result.push(symbol.location);
            }
        }
        // only return results that are of the same language unless there are only 
        // results from other languages
        return result.slice(0, sameLanguageOffset || undefined);
    }
}
exports.DefinitionProvider = DefinitionProvider;
