"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DocumentHighlightsProvider = void 0;
const lsp = __importStar(require("vscode-languageserver"));
const locals_1 = require("./locals");
const languages_1 = __importDefault(require("../languages"));
const common_1 = require("../common");
class DocumentHighlightsProvider {
    constructor(_documents, _trees) {
        this._documents = _documents;
        this._trees = _trees;
    }
    register(connection) {
        connection.client.register(lsp.DocumentHighlightRequest.type, { documentSelector: languages_1.default.getSupportedLanguages('highlights', ['locals', 'identifiers']) });
        connection.onRequest(lsp.DocumentHighlightRequest.type, this.provideDocumentHighlights.bind(this));
    }
    async provideDocumentHighlights(params) {
        const document = await this._documents.retrieve(params.textDocument.uri);
        const info = locals_1.Locals.create(document, this._trees);
        const scope = info.root.findScope(params.position);
        const anchor = scope.findDefinitionOrUsage(params.position);
        if (!anchor) {
            return this._identifierBasedHighlights(document, params.position);
        }
        const result = [];
        for (let def of scope.findDefinitions(anchor.name)) {
            result.push(lsp.DocumentHighlight.create(def.range, lsp.DocumentHighlightKind.Write));
        }
        if (result.length === 0) {
            // needs a definition
            return this._identifierBasedHighlights(document, params.position);
        }
        for (let usage of scope.findUsages(anchor.name)) {
            result.push(lsp.DocumentHighlight.create(usage.range, lsp.DocumentHighlightKind.Read));
        }
        return result;
    }
    _identifierBasedHighlights(document, position) {
        const result = [];
        const tree = this._trees.getParseTree(document);
        if (!tree) {
            return result;
        }
        const query = languages_1.default.getQuery(document.languageId, 'identifiers');
        const candidate = common_1.identifierAtPosition(query, tree.rootNode, position);
        if (!candidate) {
            // not on an identifier
            return result;
        }
        for (let capture of query.captures(tree.rootNode)) {
            // same node text, e.g foo vs bar
            if (capture.node.text === candidate.text) {
                result.push(lsp.DocumentHighlight.create(common_1.asLspRange(capture.node), lsp.DocumentHighlightKind.Text));
            }
        }
        return result;
    }
}
exports.DocumentHighlightsProvider = DocumentHighlightsProvider;
