"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Outline = exports.DocumentSymbols = void 0;
const lsp = __importStar(require("vscode-languageserver"));
const common_1 = require("../common");
const languages_1 = __importDefault(require("../languages"));
class DocumentSymbols {
    constructor(_documents, _trees) {
        this._documents = _documents;
        this._trees = _trees;
    }
    register(connection) {
        connection.client.register(lsp.DocumentSymbolRequest.type, { documentSelector: languages_1.default.getSupportedLanguages('outline', ['outline']) });
        connection.onRequest(lsp.DocumentSymbolRequest.type, this.provideDocumentSymbols.bind(this));
    }
    async provideDocumentSymbols(params) {
        const document = await this._documents.retrieve(params.textDocument.uri);
        return Outline.create(document, this._trees);
    }
}
exports.DocumentSymbols = DocumentSymbols;
class Node {
    constructor(capture) {
        this.capture = capture;
        this.children = [];
        this.range = common_1.asLspRange(capture.node);
    }
}
class Outline {
    static create(document, trees) {
        const tree = trees.getParseTree(document);
        if (!tree) {
            return [];
        }
        const query = languages_1.default.getQuery(document.languageId, 'outline');
        const captures = query.captures(tree.rootNode);
        // build a Node-tree that is based on range containment. This includes true 
        // children as well as the "name-child"
        const roots = [];
        const stack = [];
        for (const capture of captures) {
            const node = new Node(capture);
            let parent = stack.pop();
            while (true) {
                if (!parent) {
                    roots.push(node);
                    stack.push(node);
                    break;
                }
                if (common_1.containsRange(parent.range, node.range)) {
                    parent.children.push(node);
                    stack.push(parent);
                    stack.push(node);
                    break;
                }
                parent = stack.pop();
            }
        }
        // build DocumentSymbol-tree from Node-tree. Children of nodes that match
        // the `<xyz>.name` capture name are used as identifier/name and aren't producing
        // a dedicated document symbol
        function build(node, bucket) {
            let children = [];
            let nameNode;
            for (let child of node.children) {
                if (!nameNode && child.capture.name.endsWith('.name') && child.capture.name.startsWith(node.capture.name)) {
                    nameNode = child;
                }
                else {
                    build(child, children);
                }
            }
            if (!nameNode) {
                nameNode = node;
            }
            const symbol = lsp.DocumentSymbol.create(nameNode.capture.node.text, '', common_1.symbolMapping.getSymbolKind(node.capture.name), node.range, nameNode.range);
            symbol.children = children;
            bucket.push(symbol);
        }
        const result = [];
        for (let node of roots) {
            build(node, result);
        }
        return result;
    }
}
exports.Outline = Outline;
