"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.FoldingRangeProvider = void 0;
const lsp = __importStar(require("vscode-languageserver"));
const languages_1 = __importDefault(require("../languages"));
class FoldingRangeProvider {
    constructor(_documents, _trees) {
        this._documents = _documents;
        this._trees = _trees;
    }
    register(connection) {
        connection.client.register(lsp.FoldingRangeRequest.type, { documentSelector: languages_1.default.getSupportedLanguages('folding', ['comments', 'folding']) });
        connection.onRequest(lsp.FoldingRangeRequest.type, this.provideFoldingRanges.bind(this));
    }
    async provideFoldingRanges(params) {
        const document = await this._documents.retrieve(params.textDocument.uri);
        const tree = this._trees.getParseTree(document);
        if (!tree) {
            return [];
        }
        const result = [];
        const commentQuery = languages_1.default.getQuery(document.languageId, 'comments');
        const commentCaptures = commentQuery.captures(tree.rootNode);
        const foldingQuery = languages_1.default.getQuery(document.languageId, 'folding');
        const foldingCaptures = foldingQuery.captures(tree.rootNode);
        for (const capture of [commentCaptures, foldingCaptures].flat()) {
            result.push(lsp.FoldingRange.create(capture.node.startPosition.row, capture.node.endPosition.row, capture.node.startPosition.column, capture.node.endPosition.column, capture.name));
        }
        return result;
    }
}
exports.FoldingRangeProvider = FoldingRangeProvider;
