"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ReferencesProvider = void 0;
const lsp = __importStar(require("vscode-languageserver"));
const locals_1 = require("./locals");
const languages_1 = __importDefault(require("../languages"));
const common_1 = require("../common");
class ReferencesProvider {
    constructor(_documents, _trees, _symbols) {
        this._documents = _documents;
        this._trees = _trees;
        this._symbols = _symbols;
    }
    register(connection) {
        connection.client.register(lsp.ReferencesRequest.type, { documentSelector: languages_1.default.getSupportedLanguages('references', ['locals', 'identifiers', 'references']) });
        connection.onRequest(lsp.ReferencesRequest.type, this.provideReferences.bind(this));
    }
    async provideReferences(params) {
        const document = await this._documents.retrieve(params.textDocument.uri);
        // find references inside file
        const info = locals_1.Locals.create(document, this._trees);
        const scope = info.root.findScope(params.position);
        const anchor = scope.findDefinitionOrUsage(params.position);
        if (!scope.likelyExports && anchor) {
            const definitions = scope.findDefinitions(anchor.name);
            if (definitions.length > 0) {
                const result = [];
                for (let def of definitions) {
                    if (params.context.includeDeclaration) {
                        result.push(lsp.Location.create(document.uri, def.range));
                    }
                }
                const usages = scope.findUsages(anchor.name);
                for (let usage of usages) {
                    result.push(lsp.Location.create(document.uri, usage.range));
                }
                return result;
            }
        }
        // find references globally
        return await this._findGlobalReferences(document, params.position, params.context.includeDeclaration);
    }
    async _findGlobalReferences(document, position, includeDeclaration) {
        const tree = this._trees.getParseTree(document);
        if (!tree) {
            return [];
        }
        const query = languages_1.default.getQuery(document.languageId, 'identifiers');
        const candidate = common_1.identifierAtPosition(query, tree.rootNode, position);
        if (!candidate) {
            // not an identifier
            return [];
        }
        await this._symbols.update();
        let sameLanguageOffset = 0;
        const result = [];
        let seenAsUsage = false;
        let seenAsDef = false;
        const usages = this._symbols.usages.get(candidate.text) ?? [];
        for (let usage of usages) {
            seenAsUsage = seenAsUsage || common_1.containsPosition(usage.range, position);
            if (languages_1.default.getLanguageIdByUri(usage.uri) === document.languageId) {
                result.unshift(usage);
                sameLanguageOffset++;
            }
            else {
                result.push(usage);
            }
        }
        const definitions = this._symbols.definitions.get(candidate.text) ?? [];
        for (const { location } of definitions) {
            seenAsDef = seenAsDef || common_1.containsPosition(location.range, position);
            if (includeDeclaration) {
                if (languages_1.default.getLanguageIdByUri(location.uri) === document.languageId) {
                    result.unshift(location);
                    sameLanguageOffset++;
                }
                else {
                    result.push(location);
                }
            }
        }
        if (!seenAsUsage && !seenAsDef) {
            // flishy results because we didn't see the location at which we requested references
            return [];
        }
        // only return results that are of the same language unless there are only 
        // results from other languages
        return result.slice(0, sameLanguageOffset || undefined);
    }
}
exports.ReferencesProvider = ReferencesProvider;
