"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const assert_1 = __importDefault(require("assert"));
const common_1 = require("../common");
const documentHighlights_1 = require("../features/documentHighlights");
const trees_1 = require("../trees");
const utils_1 = require("./utils");
suite('DocumentHighlight - Fixtures', function () {
    suiteSetup(async function () {
        await utils_1.bootstrapWasm();
    });
    function assertDocumentHighlights(fixture, actual) {
        actual.sort((a, b) => common_1.compareRangeByStart(a.range, b.range));
        if (actual.length === 0) {
            assert_1.default.fail('NO symbols found: ' + fixture.name);
        }
        for (let highlight of actual) {
            const e = fixture.marks.shift();
            assert_1.default.ok(e);
            assert_1.default.strictEqual(fixture.document.offsetAt(highlight.range.start), e.start, fixture.name);
        }
        if (fixture.marks.length > 0) {
            assert_1.default.fail('not ALL MARKS seen: ' + fixture.name);
        }
    }
    ['go', 'java', 'rust', 'csharp', 'php'].forEach(async (langId) => {
        test(langId, async function () {
            const fixtures = await utils_1.Fixture.parse(`/server/src/test/documentHighlightsFixtures/${langId}.txt`, langId);
            const store = new utils_1.TestDocumentStore(...fixtures.map(f => f.document));
            const trees = new trees_1.Trees(store);
            for (let item of fixtures) {
                const symbols = new documentHighlights_1.DocumentHighlightsProvider(store, trees);
                const result = await symbols.provideDocumentHighlights({
                    textDocument: { uri: item.document.uri },
                    position: item.document.positionAt(item.marks[0].start)
                });
                assertDocumentHighlights(item, result);
            }
        });
    });
});
