"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const assert_1 = __importDefault(require("assert"));
const documentSymbols_1 = require("../features/documentSymbols");
const trees_1 = require("../trees");
const utils_1 = require("./utils");
suite('DocumentSymbols - Fixtures', function () {
    suiteSetup(async function () {
        await utils_1.bootstrapWasm();
    });
    function assertDocumentSymbols(fixture, actual) {
        if (actual.length === 0) {
            assert_1.default.fail('NO symbols found');
        }
        (function walk(symbols) {
            for (let symbol of symbols) {
                const expected = fixture.marks.shift();
                assert_1.default.ok(expected, `symbol NOT expected: ${symbol.name}@${symbol.range.start.line},${symbol.range.start.character}`);
                assert_1.default.strictEqual(symbol.name, expected.text);
                assert_1.default.strictEqual(fixture.document.offsetAt(symbol.selectionRange.start), expected.start);
                if (symbol.children) {
                    walk(symbol.children);
                }
            }
        })(actual);
        if (fixture.marks.length > 0) {
            assert_1.default.fail(`also EXPECTED ${fixture.marks.map(e => e.text)}`);
        }
    }
    ['go', 'java', 'python', 'rust', 'csharp', 'typescript', 'c'].forEach(async (langId) => {
        test(langId, async function () {
            const fixtures = await utils_1.Fixture.parse(`/server/src/test/documentSymbolsFixtures/${langId}.txt`, langId);
            const store = new utils_1.TestDocumentStore(...fixtures.map(f => f.document));
            const trees = new trees_1.Trees(store);
            for (let item of fixtures) {
                const symbols = new documentSymbols_1.DocumentSymbols(store, trees);
                const result = await symbols.provideDocumentSymbols({ textDocument: { uri: item.document.uri } });
                assertDocumentSymbols(item, result);
            }
        });
    });
});
