"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const assert_1 = __importDefault(require("assert"));
const trie_1 = require("../util/trie");
suite('Trie', function () {
    function assertKeys(trie, keys) {
        const actual = [];
        for (let item of trie) {
            actual.push(item[0]);
        }
        assert_1.default.deepStrictEqual(actual, keys);
    }
    test('set/get/delete', function () {
        const trie = trie_1.Trie.create();
        trie.set('aa', true);
        trie.set('aaa', true);
        trie.set('foo', true);
        assertKeys(trie, ['aa', 'aaa', 'foo']);
        trie.set('Foo', true);
        trie.set('Foooo', true);
        assertKeys(trie, ['aa', 'aaa', 'foo', 'Foo', 'Foooo']);
        trie.delete('aa');
        assertKeys(trie, ['aaa', 'foo', 'Foo', 'Foooo']);
        trie.delete('Foooo');
        assertKeys(trie, ['aaa', 'foo', 'Foo']);
    });
    test('depth', function () {
        const trie = trie_1.Trie.create();
        trie.set('aa', true);
        assert_1.default.strictEqual(trie.depth, 2);
        trie.set('foo', true);
        assert_1.default.strictEqual(trie.depth, 3);
        assert_1.default.ok(trie.delete('foo'));
        assert_1.default.strictEqual(trie.depth, 2);
        trie.set('aaaa', true);
        assert_1.default.strictEqual(trie.depth, 4);
        assert_1.default.ok(trie.delete('aa'));
        assert_1.default.strictEqual(trie.depth, 4);
        trie.set('aaa', true);
        assert_1.default.strictEqual(trie.depth, 4);
        assert_1.default.ok(trie.delete('aaaa'));
        assert_1.default.strictEqual(trie.depth, 3);
        assertKeys(trie, ['aaa']);
    });
    test('query', function () {
        const trie = trie_1.Trie.create();
        trie.set('foo', true);
        trie.set('Foo', true);
        trie.set('barFoo', true);
        let arr = Array.from(trie.query(Array.from('fo')));
        assert_1.default.strictEqual(arr.length, 2);
        arr = Array.from(trie.query(Array.from('b')));
        assert_1.default.strictEqual(arr.length, 1);
        arr = Array.from(trie.query(Array.from('foobar')));
        assert_1.default.strictEqual(arr.length, 0);
    });
});
