"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Fixture = exports.FixtureMarks = exports.TestDocumentStore = exports.mock = exports.bootstrapWasm = void 0;
const tree_sitter_1 = __importDefault(require("../../tree-sitter/tree-sitter"));
const languages_1 = __importDefault(require("../languages"));
const documentStore_1 = require("../documentStore");
const vscode_languageserver_textdocument_1 = require("vscode-languageserver-textdocument");
async function bootstrapWasm() {
    await tree_sitter_1.default.init({
        locateFile() {
            return '/server/tree-sitter/tree-sitter.wasm';
        }
    });
    await languages_1.default.init(new Map([
        [{ languageId: 'csharp', wasmUri: '/server/tree-sitter-c_sharp.wasm', suffixes: [] }, {}],
        [{ languageId: 'c', wasmUri: '/server/tree-sitter-c.wasm', suffixes: [] }, {}],
        [{ languageId: 'cpp', wasmUri: '/server/tree-sitter-cpp.wasm', suffixes: [] }, {}],
        [{ languageId: 'go', wasmUri: '/server/tree-sitter-go.wasm', suffixes: [] }, {}],
        [{ languageId: 'java', wasmUri: '/server/tree-sitter-java.wasm', suffixes: [] }, {}],
        [{ languageId: 'php', wasmUri: '/server/tree-sitter-php.wasm', suffixes: [] }, {}],
        [{ languageId: 'python', wasmUri: '/server/tree-sitter-python.wasm', suffixes: [] }, {}],
        [{ languageId: 'rust', wasmUri: '/server/tree-sitter-rust.wasm', suffixes: [] }, {}],
        [{ languageId: 'typescript', wasmUri: '/server/tree-sitter-typescript.wasm', suffixes: [] }, {}],
    ]));
}
exports.bootstrapWasm = bootstrapWasm;
function mock() {
    return function () { };
}
exports.mock = mock;
class TestDocumentStore extends documentStore_1.DocumentStore {
    constructor(...docs) {
        super(new class extends mock() {
            onDidOpenTextDocument(handler) {
                for (let doc of docs) {
                    handler({
                        textDocument: {
                            languageId: doc.languageId,
                            uri: doc.uri,
                            text: doc.getText(),
                            version: doc.version
                        }
                    });
                }
            }
            onDidChangeTextDocument() { }
            onDidCloseTextDocument() { }
            onWillSaveTextDocument() { }
            onWillSaveTextDocumentWaitUntil() { }
            onDidSaveTextDocument() { }
            onNotification() { }
        });
    }
}
exports.TestDocumentStore = TestDocumentStore;
class FixtureMarks {
    constructor(start, text) {
        this.start = start;
        this.text = text;
    }
}
exports.FixtureMarks = FixtureMarks;
FixtureMarks.pattern = /\[\[[^\]]+\]\]/g;
class Fixture {
    constructor(name, document, marks) {
        this.name = name;
        this.document = document;
        this.marks = marks;
    }
    static async parse(uri, languageId) {
        const res = await fetch(uri);
        const text = await res.text();
        const result = [];
        const r = /---.*/gu;
        const fixtures = text.split(r).filter(Boolean);
        const matches = text.match(r);
        for (let i = 0; i < fixtures.length; i++) {
            let text = fixtures[i];
            const marks = [];
            const parts = text.split(FixtureMarks.pattern);
            const idents = text.match(FixtureMarks.pattern);
            text = '';
            for (let i = 0; i < parts.length; i++) {
                text += parts[i];
                let ident = idents[i];
                if (ident) {
                    let name = ident.slice(2, -2);
                    marks.push(new FixtureMarks(text.length, name));
                    text += name;
                }
            }
            const name = matches?.shift()?.substring(3) ?? String(i);
            if (name.includes('/SKIP/')) {
                continue;
            }
            result.push(new Fixture(name, vscode_languageserver_textdocument_1.TextDocument.create(`${uri}#${i}`, languageId, 1, text), marks));
        }
        return result;
    }
}
exports.Fixture = Fixture;
;
